@@ -543,7 +543,6 @@ ORBit_demarshal_value(CORBA_TypeCode tc,
     *val = ALIGN_ADDRESS(*val, ORBIT_ALIGNOF_CORBA_POINTER);
     if(ORBit_decode_CORBA_TypeCode(*val, buf))
       return TRUE;
-    ORBit_RootObject_duplicate(*(CORBA_TypeCode *)*val);
     *val = ((guchar *)*val) + sizeof(CORBA_TypeCode);
     break;
   case CORBA_tk_except:
@@ -652,62 +651,75 @@ ORBit_demarshal_value(CORBA_TypeCode tc,
 
 
 gpointer
-ORBit_demarshal_arg(GIOPRecvBuffer *buf,
-		    CORBA_TypeCode tc,
-		    gboolean dup_strings,
-		    CORBA_ORB orb)
+ORBit_demarshal_arg (GIOPRecvBuffer *buf,
+		     CORBA_TypeCode tc,
+		     gboolean       dup_strings,
+		     CORBA_ORB      orb)
 {
-  gpointer retval, val;
+	gpointer retval, val;
 
-  retval = val = ORBit_alloc_tcval(tc, 1);
+	retval = val = ORBit_alloc_tcval (tc, 1);
 
-  ORBit_demarshal_value(tc, &val, buf, dup_strings, orb);
+	ORBit_demarshal_value (tc, &val, buf, dup_strings, orb);
 
-  return retval;
+	return retval;
 }
 
 gboolean
-ORBit_demarshal_any(GIOPRecvBuffer *buf, CORBA_any *retval,
-		    gboolean dup_strings,
-		    CORBA_ORB orb)
+ORBit_demarshal_any (GIOPRecvBuffer *buf,
+		     CORBA_any      *retval,
+		     gboolean        dup_strings,
+		     CORBA_ORB       orb)
 {
-  gpointer val;
+	gpointer val;
 
-  CORBA_any_set_release(retval, CORBA_TRUE);
+	CORBA_any_set_release (retval, CORBA_TRUE);
 
-  if(ORBit_decode_CORBA_TypeCode(&retval->_type, buf))
-    return TRUE;
-  ORBit_RootObject_duplicate(retval->_type);
+	if (ORBit_decode_CORBA_TypeCode (&retval->_type, buf))
+		return TRUE;
 
-  val = retval->_value = ORBit_alloc_tcval(retval->_type, 1);
-  if(ORBit_demarshal_value(retval->_type, &val, buf, dup_strings, orb))
-    return TRUE;
-  return FALSE;
+	val = retval->_value = ORBit_alloc_tcval (retval->_type, 1);
+	if (ORBit_demarshal_value (retval->_type, &val,
+				   buf, dup_strings, orb))
+		return TRUE;
+
+	return FALSE;
 }
 
 gpointer
-CORBA_any__freekids(gpointer mem, gpointer dat)
+CORBA_any__freekids (gpointer mem, gpointer dat)
 {
-  CORBA_any *t;
-  t = mem;
-  if(t->_type)
-    ORBit_RootObject_release_T ((ORBit_RootObject)t->_type);
-  if(t->_release)
-    ORBit_free_T(t->_value);
-  return t + 1;
+	CORBA_any *t;
+
+	t = mem;
+	if (t->_type)
+		ORBit_RootObject_release_T (
+			(ORBit_RootObject) t->_type);
+
+	if (t->_release)
+		ORBit_free_T (t->_value);
+
+	return t + 1;
 }
 
 CORBA_any *
-CORBA_any__alloc(void)
+CORBA_any__alloc (void)
 {
-  CORBA_any *retval = ORBit_alloc(sizeof(CORBA_any), 1, 
-				  &CORBA_any__freekids);
-  memset(retval, 0, sizeof(CORBA_any)); /* Make things easier on stubs */
-  return retval;
+	CORBA_any *retval = ORBit_alloc (
+		sizeof(CORBA_any), 1, 
+		&CORBA_any__freekids);
+
+	/* FIXME: do we need to do this for small ? */
+	/* Make things easier on stubs */
+	memset (retval, 0, sizeof (CORBA_any));
+
+	return retval;
 }
 
 void
-ORBit_copy_value_core(gconstpointer *val, gpointer *newval, CORBA_TypeCode tc)
+ORBit_copy_value_core (gconstpointer *val,
+		       gpointer      *newval,
+		       CORBA_TypeCode tc)
 {
   CORBA_long i;
   gconstpointer pval1; 
@@ -887,22 +899,22 @@ ORBit_copy_value_core(gconstpointer *val, gpointer *newval, CORBA_TypeCode tc)
 }
 
 gpointer
-ORBit_copy_value(gconstpointer value, CORBA_TypeCode tc)
+ORBit_copy_value (gconstpointer value, CORBA_TypeCode tc)
 {
-  gpointer retval, newval;
+	gpointer retval, newval;
 
-  retval = newval = ORBit_alloc_tcval(tc, 1);
-  ORBit_copy_value_core(&value, &newval, tc);
+	retval = newval = ORBit_alloc_tcval (tc, 1);
+	ORBit_copy_value_core (&value, &newval, tc);
 
-  return retval;
+	return retval;
 }
 
 void
-CORBA_any__copy(CORBA_any *out, const CORBA_any *in)
+CORBA_any__copy (CORBA_any *out, const CORBA_any *in)
 {
-  out->_type = ORBit_RootObject_duplicate(in->_type);
-  out->_value = ORBit_copy_value(in->_value, in->_type);
-  out->_release = CORBA_TRUE;
+	out->_type = ORBit_RootObject_duplicate (in->_type);
+	out->_value = ORBit_copy_value (in->_value, in->_type);
+	out->_release = CORBA_TRUE;
 }
 
 #define ALIGN_COMPARE(a,b,tk,type,align)	\
