@@ -9,6 +9,10 @@
 
 #undef DEBUG
 
+/* FIXME: this whole module is horribly unneccesary - if we just
+   used the wire structures more intelligently we could store
+   everything far more sensibly */
+
 /*
  * common set of profiles for Ojbect Adaptor generated refs
  */
@@ -38,7 +42,7 @@ IOP_ObjectKey_dump (ORBit_ObjectKey *objkey)
 	GString *str = g_string_sized_new (objkey->_length * 2 + 4);
 
 	for (i = 0; i < objkey->_length; i++)
-		g_string_printfa (str, "%2x", objkey->_buffer [i]);
+		g_string_append_printf (str, "%2x", objkey->_buffer [i]);
 
 	return g_string_free (str, FALSE);
 }
@@ -866,46 +870,52 @@ IOP_components_marshal (CORBA_Object obj,
 }
 
 static void
-IOP_TAG_INTERNET_IOP_marshal(CORBA_Object obj, GIOPSendBuffer *buf, 
-			     IOP_Profile_info *profile)
+IOP_TAG_INTERNET_IOP_marshal (CORBA_Object      obj,
+			      GIOPSendBuffer   *buf, 
+			      IOP_Profile_info *profile)
 {
-  IOP_TAG_INTERNET_IOP_info *iiop = (IOP_TAG_INTERNET_IOP_info *)profile;
-  CORBA_unsigned_long len;
+	IOP_TAG_INTERNET_IOP_info *iiop;
+	CORBA_unsigned_long len;
 
-  giop_send_buffer_append(buf, giop_version_ids[iiop->iiop_version], 2);
-  len = strlen(iiop->host) + 1;
-  giop_send_buffer_append_aligned(buf, &len, 4);
-  giop_send_buffer_append(buf, iiop->host, len);
+	iiop = (IOP_TAG_INTERNET_IOP_info *) profile;
 
-  giop_send_buffer_align(buf, 2);
-  giop_send_buffer_append(buf, &iiop->port, 2);
+	giop_send_buffer_append (
+		buf, giop_version_ids [iiop->iiop_version], 2);
 
-  IOP_ObjectKey_marshal(obj, buf, obj->object_key);
+	len = strlen (iiop->host) + 1;
+	giop_send_buffer_append_aligned (buf, &len, 4);
+	giop_send_buffer_append (buf, iiop->host, len);
 
-  IOP_components_marshal(obj, buf, iiop->components);
+	giop_send_buffer_align (buf, 2);
+	giop_send_buffer_append (buf, &iiop->port, 2);
+
+	IOP_ObjectKey_marshal (obj, buf, obj->object_key);
+
+	IOP_components_marshal (obj, buf, iiop->components);
 }
 
 static void
-IOP_TAG_GENERIC_IOP_marshal(CORBA_Object obj, GIOPSendBuffer *buf, 
-			    IOP_Profile_info *profile)
+IOP_TAG_GENERIC_IOP_marshal (CORBA_Object     obj,
+			     GIOPSendBuffer   *buf, 
+			     IOP_Profile_info *profile)
 {
-  IOP_TAG_GENERIC_IOP_info *giop = (IOP_TAG_GENERIC_IOP_info *)profile;
-  CORBA_unsigned_long len;
+	IOP_TAG_GENERIC_IOP_info *giop = (IOP_TAG_GENERIC_IOP_info *) profile;
+	CORBA_unsigned_long       len;
 
-  giop_send_buffer_append(buf, giop_version_ids[giop->iiop_version], 2);
-  len = strlen(giop->proto) + 1;
-  giop_send_buffer_append_aligned(buf, &len, 4);
-  giop_send_buffer_append(buf, giop->proto, len);
+	giop_send_buffer_append (buf, giop_version_ids [giop->iiop_version], 2);
+	len = strlen (giop->proto) + 1;
+	giop_send_buffer_append_aligned (buf, &len, 4);
+	giop_send_buffer_append (buf, giop->proto, len);
 
-  len = strlen(giop->host) + 1;
-  giop_send_buffer_append_aligned(buf, &len, 4);
-  giop_send_buffer_append(buf, giop->host, len);
+	len = strlen (giop->host) + 1;
+	giop_send_buffer_append_aligned (buf, &len, 4);
+	giop_send_buffer_append (buf, giop->host, len);
 
-  len = strlen(giop->service) + 1;
-  giop_send_buffer_append_aligned(buf, &len, 4);
-  giop_send_buffer_append(buf, giop->service, len);
+	len = strlen (giop->service) + 1;
+	giop_send_buffer_append_aligned (buf, &len, 4);
+	giop_send_buffer_append (buf, giop->service, len);
 
-  IOP_components_marshal(obj, buf, giop->components);
+	IOP_components_marshal (obj, buf, giop->components);
 }
 
 static void
@@ -913,34 +923,44 @@ IOP_TAG_MULTIPLE_COMPONENTS_marshal (CORBA_Object      obj,
 				     GIOPSendBuffer   *buf,
 				     IOP_Profile_info *profile)
 {
-	IOP_TAG_MULTIPLE_COMPONENTS_info *mci = (IOP_TAG_MULTIPLE_COMPONENTS_info*)
-		profile;
+	IOP_TAG_MULTIPLE_COMPONENTS_info *mci;
+
+	mci = (IOP_TAG_MULTIPLE_COMPONENTS_info*) profile;
 
 	IOP_components_marshal(obj, buf, mci->components);
 }
 
 static void
-IOP_TAG_ORBIT_SPECIFIC_marshal(CORBA_Object obj, GIOPSendBuffer *buf,
-			       IOP_Profile_info *profile)
+IOP_TAG_ORBIT_SPECIFIC_marshal (CORBA_Object      obj,
+				GIOPSendBuffer   *buf,
+				IOP_Profile_info *profile)
 {
-  CORBA_unsigned_long len;
-  IOP_TAG_ORBIT_SPECIFIC_info *osi = (IOP_TAG_ORBIT_SPECIFIC_info*)profile;
+	CORBA_unsigned_long len;
+	IOP_TAG_ORBIT_SPECIFIC_info *osi;
+	guchar compat_version[2] = { 1, 2 }; /* for ORBit1 */
 
-  len = strlen(osi->unix_sock_path) + 1;
-  giop_send_buffer_append_aligned(buf, &len, 4);
-  giop_send_buffer_append(buf, osi->unix_sock_path, len);
-  giop_send_buffer_align(buf, 2);
-  giop_send_buffer_append(buf, &osi->ipv6_port, 2);
-  IOP_ObjectKey_marshal(obj, buf, obj->object_key);
+	osi = (IOP_TAG_ORBIT_SPECIFIC_info*) profile;
+
+	giop_send_buffer_append (buf, compat_version, 2);
+
+	len = strlen (osi->unix_sock_path) + 1;
+	giop_send_buffer_append_aligned (buf, &len, 4);
+	giop_send_buffer_append (buf, osi->unix_sock_path, len);
+
+	giop_send_buffer_align (buf, 2);
+	giop_send_buffer_append (buf, &osi->ipv6_port, 2);
+
+	IOP_ObjectKey_marshal (obj, buf, obj->object_key);
 }
 
 static void
 IOP_UnknownProfile_marshal(CORBA_Object obj, GIOPSendBuffer *buf, 
 			   IOP_Profile_info *pi)
 {
-  IOP_UnknownProfile_info *upi = (IOP_UnknownProfile_info *)pi;
+	IOP_UnknownProfile_info *upi = (IOP_UnknownProfile_info *) pi;
 
-  giop_send_buffer_append(buf, upi->data._buffer, upi->data._length);
+	giop_send_buffer_append (
+		buf, upi->data._buffer, upi->data._length);
 }
 
 void
@@ -985,9 +1005,6 @@ IOP_profile_marshal (CORBA_Object obj, GIOPSendBuffer *buf, gpointer *p)
  * demarshalling routines.
  */
 
-/* FIXME: WTF ! */
-static GIOPRecvBuffer *gbuf = NULL;
-
 static ORBit_ObjectKey*
 IOP_ObjectKey_demarshal (GIOPRecvBuffer *buf)
 {
@@ -1023,6 +1040,24 @@ IOP_ObjectKey_demarshal (GIOPRecvBuffer *buf)
 	return objkey;
 }
 
+ORBit_ObjectKey *
+IOP_ObjectKey_copy (ORBit_ObjectKey *src)
+{
+	ORBit_ObjectKey *objkey;
+
+	if (!src)
+		return NULL;
+
+	objkey           = CORBA_sequence_CORBA_octet__alloc ();
+	objkey->_length  = objkey->_maximum = src->_length;
+	objkey->_buffer  = CORBA_sequence_CORBA_octet_allocbuf (src->_length);
+	objkey->_release = CORBA_TRUE;
+
+	memcpy (objkey->_buffer, src->_buffer, src->_length);
+
+	return objkey;
+}
+
 static IOP_Component_info *
 IOP_TAG_SSL_SEC_TRANS_demarshal(IOP_ComponentId id, GIOPRecvBuffer *buf)
 {
@@ -1278,75 +1313,89 @@ IOP_UnknownProfile_demarshal(IOP_ProfileId p, GIOPRecvBuffer *buf,
 }
 
 static IOP_Profile_info *
-IOP_TAG_ORBIT_SPECIFIC_demarshal(IOP_ProfileId p, GIOPRecvBuffer *pbuf,
-				 CORBA_ORB orb)
+IOP_TAG_ORBIT_SPECIFIC_demarshal (IOP_ProfileId   p,
+				  GIOPRecvBuffer *pbuf,
+				  CORBA_ORB       orb)
 {
-  IOP_TAG_ORBIT_SPECIFIC_info *retval = NULL;
-  CORBA_unsigned_long len;
-  GIOPRecvBuffer *buf;
+	IOP_TAG_ORBIT_SPECIFIC_info *retval = NULL;
+	CORBA_unsigned_long          len;
+	GIOPRecvBuffer              *buf;
+	gboolean                     msg_conversion;
 
-  buf = giop_recv_buffer_use_encaps_buf(pbuf);
-  if(!buf)
-    goto errout;
+	buf = giop_recv_buffer_use_encaps_buf (pbuf);
+	if (!buf)
+		goto errout;
 
-  buf->cur = ALIGN_ADDRESS(buf->cur, 4);
-  if((buf->cur + 4) > buf->end)
-    return NULL;
-  len = *(CORBA_unsigned_long *)buf->cur;
-  if(giop_msg_conversion_needed(buf))
-    len = GUINT32_SWAP_LE_BE(len);
-  buf->cur += 4;
-  if((buf->cur + len) > buf->end
-     || (buf->cur + len) < buf->cur)
-    goto errout;
+	/* Ignore the version info */
+	if ((buf->cur + 2) > buf->end)
+		return NULL;
+	buf->cur += 2;
 
-  retval = g_new(IOP_TAG_ORBIT_SPECIFIC_info, 1);
-  retval->parent.profile_type = p;
-  retval->unix_sock_path = g_malloc(len);
-  memcpy(retval->unix_sock_path, buf->cur, len);
-  buf->cur += len;
-  buf->cur = ALIGN_ADDRESS(buf->cur, 2);
-  if((buf->cur + 2) > buf->end)
-    goto errout;
-  retval->ipv6_port = *(CORBA_unsigned_short *)buf->cur;
-  if(giop_msg_conversion_needed(buf))
-    retval->ipv6_port = GUINT16_SWAP_LE_BE(retval->ipv6_port);
-  buf->cur += 2;
-  retval->object_key = IOP_ObjectKey_demarshal(buf);
-  if(!retval->object_key)
-    goto errout;
+	msg_conversion = giop_msg_conversion_needed (buf);
 
-  giop_recv_buffer_unuse(buf);
-  return (IOP_Profile_info *)retval;
+	buf->cur = ALIGN_ADDRESS (buf->cur, 4);
+	if ((buf->cur + 4) > buf->end)
+		return NULL;
+	len = *(CORBA_unsigned_long *) buf->cur;
+	if (msg_conversion)
+		len = GUINT32_SWAP_LE_BE (len);
+	buf->cur += 4;
+	if ((buf->cur + len) > buf->end || (buf->cur + len) < buf->cur)
+		goto errout;
+
+	retval = g_new (IOP_TAG_ORBIT_SPECIFIC_info, 1);
+	retval->parent.profile_type = p;
+	retval->unix_sock_path = g_malloc (len);
+	memcpy (retval->unix_sock_path, buf->cur, len);
+	buf->cur += len;
+
+	buf->cur = ALIGN_ADDRESS (buf->cur, 2);
+	if ((buf->cur + 2) > buf->end)
+		goto errout;
+
+	retval->ipv6_port = *(CORBA_unsigned_short *) buf->cur;
+	if (msg_conversion)
+		retval->ipv6_port = GUINT16_SWAP_LE_BE (retval->ipv6_port);
+	buf->cur += 2;
+
+	retval->object_key = IOP_ObjectKey_demarshal (buf);
+	if (!retval->object_key)
+		goto errout;
+
+	giop_recv_buffer_unuse (buf);
+
+	return (IOP_Profile_info *) retval;
 
  errout:
-  if(retval)
-    {
-      CORBA_free(retval->object_key);
-      g_free(retval->unix_sock_path);
-      g_free(retval);
-    }
-  giop_recv_buffer_unuse(buf);
-  return NULL;
+	if (retval) {
+		CORBA_free (retval->object_key);
+		g_free (retval->unix_sock_path);
+		g_free (retval);
+	}
+	giop_recv_buffer_unuse (buf);
+
+	return NULL;
 }
 
 static IOP_Profile_info *
-IOP_TAG_MULTIPLE_COMPONENTS_demarshal(IOP_ProfileId p, GIOPRecvBuffer *pbuf,
-				      CORBA_ORB orb)
+IOP_TAG_MULTIPLE_COMPONENTS_demarshal (IOP_ProfileId   p,
+				       GIOPRecvBuffer *pbuf,
+				       CORBA_ORB       orb)
 {
-  IOP_TAG_MULTIPLE_COMPONENTS_info *retval = NULL;
-  GIOPRecvBuffer *buf;
-  GSList *components;
+	IOP_TAG_MULTIPLE_COMPONENTS_info *retval = NULL;
+	GIOPRecvBuffer *buf;
+	GSList         *components;
 
-  buf = giop_recv_buffer_use_encaps_buf(pbuf);
-  if(buf && !IOP_components_demarshal(buf, &components))
-    {
-      retval = g_new(IOP_TAG_MULTIPLE_COMPONENTS_info, 1);
-      retval->parent.profile_type = p;
-      retval->components = components;
-    }
-  giop_recv_buffer_unuse(buf);
-  return (IOP_Profile_info*)retval;
+	buf = giop_recv_buffer_use_encaps_buf (pbuf);
+
+	if (buf && !IOP_components_demarshal (buf, &components)) {
+		retval = g_new (IOP_TAG_MULTIPLE_COMPONENTS_info, 1);
+		retval->parent.profile_type = p;
+		retval->components = components;
+	}
+	giop_recv_buffer_unuse (buf);
+
+	return (IOP_Profile_info*) retval;
 }
 
 static IOP_Profile_info *
@@ -1594,8 +1643,6 @@ ORBit_demarshal_IOR(CORBA_ORB orb, GIOPRecvBuffer *buf,
   CORBA_unsigned_long len, num_profiles;
   int i;
 
-  gbuf = buf;
-  
   buf->cur = ALIGN_ADDRESS(buf->cur, 4);
   if((buf->cur + 4) > buf->end)
     goto errout;
@@ -1637,3 +1684,183 @@ ORBit_demarshal_IOR(CORBA_ORB orb, GIOPRecvBuffer *buf,
   IOP_delete_profiles(&profiles);
   return TRUE;
 }
+
+static void
+IOP_TAG_COMPLETE_OBJECT_KEY_copy (IOP_TAG_COMPLETE_OBJECT_KEY_info *dest,
+				  const IOP_TAG_COMPLETE_OBJECT_KEY_info *src)
+{
+	dest->object_key = IOP_ObjectKey_copy (src->object_key);
+}
+
+static void
+IOP_TAG_GENERIC_SSL_SEC_TRANS_copy (IOP_TAG_GENERIC_SSL_SEC_TRANS_info *dest,
+				    const IOP_TAG_GENERIC_SSL_SEC_TRANS_info *src)
+{
+	dest->service = g_strdup (src->service);
+}
+
+static void
+IOP_TAG_SSL_SEC_TRANS_copy (IOP_TAG_SSL_SEC_TRANS_info *dest,
+			    const IOP_TAG_SSL_SEC_TRANS_info *src)
+{
+	dest->port = src->port;
+	dest->target_supports = src->target_supports;
+	dest->target_requires = src->target_requires;
+}
+
+static void
+IOP_TAG_CODE_SETS_copy (IOP_TAG_CODE_SETS_info *dest,
+			const IOP_TAG_CODE_SETS_info *src)
+{
+	/* no-op */
+}
+
+static void
+IOP_UnknownComponent_copy (IOP_UnknownComponent_info *dest,
+			   const IOP_UnknownComponent_info *src)
+{
+	gconstpointer src_data = &src->data;
+	gpointer      dest_data = &dest->data;
+
+	ORBit_copy_value_core (
+		&src_data, &dest_data, TC_CORBA_sequence_CORBA_octet);
+}
+
+static IOP_Component_info *
+IOP_component_copy (IOP_Component_info *src)
+{
+#define CPY(src,type) \
+	G_STMT_START {  \
+	        type##_info *ret = g_new0 (type##_info, 1); \
+		ret->parent.component_type = src->component_type; \
+		type##_copy (ret, (type##_info *) src);	    \
+							    \
+	       	return (IOP_Component_info *) ret;	    \
+	} G_STMT_END
+
+	switch (src->component_type) {
+	case IOP_TAG_COMPLETE_OBJECT_KEY:
+		CPY (src, IOP_TAG_COMPLETE_OBJECT_KEY);
+		break;
+	case IOP_TAG_GENERIC_SSL_SEC_TRANS:
+		CPY (src, IOP_TAG_GENERIC_SSL_SEC_TRANS);
+		break;
+	case IOP_TAG_SSL_SEC_TRANS:
+		CPY (src, IOP_TAG_SSL_SEC_TRANS);
+		break;
+	case IOP_TAG_CODE_SETS:
+		CPY (src, IOP_TAG_CODE_SETS);
+		break;
+	default:
+		CPY (src, IOP_UnknownComponent);
+		break;
+	}
+#undef CPY
+}
+
+static GSList *
+IOP_components_copy (GSList *components)
+{
+	GSList *ret = NULL, *l;
+
+	for (l = components; l; l = l->next)
+		ret = g_slist_prepend (ret, IOP_component_copy (l->data));
+
+	return ret;
+}
+
+static void
+IOP_TAG_INTERNET_IOP_copy (IOP_TAG_INTERNET_IOP_info       *dest,
+			   const IOP_TAG_INTERNET_IOP_info *src)
+{
+	dest->iiop_version = src->iiop_version;
+	dest->port = src->port;
+	dest->object_key = IOP_ObjectKey_copy (src->object_key);
+	dest->components = IOP_components_copy (src->components);
+}
+
+static void
+IOP_TAG_GENERIC_IOP_copy (IOP_TAG_GENERIC_IOP_info       *dest,
+			  const IOP_TAG_GENERIC_IOP_info *src)
+{
+	dest->iiop_version = src->iiop_version;
+	dest->proto = g_strdup (src->proto);
+	dest->host = g_strdup (src->host);
+	dest->service = g_strdup (src->service);
+	dest->components = IOP_components_copy (src->components);
+}
+
+static void
+IOP_TAG_ORBIT_SPECIFIC_copy (IOP_TAG_ORBIT_SPECIFIC_info       *dest,
+			     const IOP_TAG_ORBIT_SPECIFIC_info *src)
+{
+	dest->unix_sock_path = g_strdup (src->unix_sock_path);
+	dest->ipv6_port = src->ipv6_port;
+	dest->object_key = IOP_ObjectKey_copy (src->object_key);
+}
+
+static void
+IOP_TAG_MULTIPLE_COMPONENTS_copy (IOP_TAG_MULTIPLE_COMPONENTS_info       *dest,
+				  const IOP_TAG_MULTIPLE_COMPONENTS_info *src)
+{
+	dest->components = IOP_components_copy (src->components);
+}
+
+static void
+IOP_UnknownProfile_copy (IOP_UnknownProfile_info *dest,
+			 const IOP_UnknownProfile_info *src)
+{
+	gconstpointer src_data = &src->data;
+	gpointer      dest_data = &dest->data;
+
+	ORBit_copy_value_core (
+		&src_data, &dest_data, TC_CORBA_sequence_CORBA_octet);
+}
+
+static IOP_Profile_info *
+IOP_profile_copy (IOP_Profile_info *src)
+{
+#define CPY(src,type) \
+	G_STMT_START {  \
+	        type##_info *ret = g_new0 (type##_info, 1); \
+		ret->parent.profile_type = src->profile_type; \
+		type##_copy (ret, (type##_info *) src);	    \
+							    \
+	       	return (IOP_Profile_info *) ret;	    \
+	} G_STMT_END
+
+	switch (src->profile_type) {
+	case IOP_TAG_INTERNET_IOP:
+		CPY (src, IOP_TAG_INTERNET_IOP);
+		break;
+
+	case IOP_TAG_MULTIPLE_COMPONENTS:
+		CPY (src, IOP_TAG_MULTIPLE_COMPONENTS);
+		break;
+
+	case IOP_TAG_GENERIC_IOP:
+		CPY (src, IOP_TAG_GENERIC_IOP);
+		break;
+
+	case IOP_TAG_ORBIT_SPECIFIC:
+		CPY (src, IOP_TAG_ORBIT_SPECIFIC);
+		break;
+
+	default:
+		CPY (src, IOP_UnknownProfile);
+		break;
+	}
+#undef CPY
+}
+
+GSList *
+IOP_profiles_copy (GSList *profile_list)
+{
+	GSList *ret = NULL;
+	GSList *l;
+
+	for (l = profile_list; l; l = l->next)
+		ret = g_slist_prepend (ret, IOP_profile_copy (l->data));
+
+	return ret;
+}
