@@ -321,11 +321,7 @@ orbit_small_marshal (CORBA_Object           obj,
 	CORBA_TypeCode           tc;
 	int                      i;
 
-#ifdef TRACE_DEBUG
-	tprintf ("p%4x : (", getpid ());
-	ORBit_trace_objref (obj);
-	tprintf (")->%s (", m_data->name);
-#endif
+	tprintf_header (obj, m_data);
 
 	{
 		int          align;
@@ -386,8 +382,6 @@ orbit_small_marshal (CORBA_Object           obj,
 
 	do_giop_dump_send (send_buffer);
 
-	giop_recv_list_setup_queue_entry (mqe, cnx, GIOP_REPLY, request_id);
-
 	if (giop_send_buffer_write (send_buffer, cnx)) {
 		giop_recv_list_destroy_queue_entry (mqe);
 		return FALSE;
@@ -601,10 +595,12 @@ ORBit_small_invoke_stub (CORBA_Object       obj,
 		goto system_exception;
 	}
 
-retry_request:
+ retry_request:
 	request_id = GPOINTER_TO_UINT (&obj);
 	completion_status = CORBA_COMPLETED_NO;
 
+	giop_recv_list_setup_queue_entry (&mqe, cnx, GIOP_REPLY, request_id);
+
 	if (!orbit_small_marshal (obj, cnx, &mqe, request_id,
 				  m_data, args, ctx))
 		goto system_exception;
@@ -613,6 +609,7 @@ retry_request:
 
 	if (m_data->flags & ORBit_I_METHOD_1_WAY) {
 		tprintf ("[ one way ]");
+		giop_recv_list_destroy_queue_entry (&mqe);
 		goto clean_out;
 	}
 
@@ -640,6 +637,7 @@ retry_request:
 		dprintf ("Clean demarshal on id 0x%x\n\n", request_id);
 		break;
 	};
+
  clean_out:
 	tprintf ("\n");
 	return;
@@ -675,6 +673,8 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 
 	has_context = (m_data->contexts._length > 0);
 
+	tprintf_header (adaptor_obj->objref, m_data);
+
 	if ((tc = m_data->ret)) {
 		
 		while (tc->kind == CORBA_tk_alias)
@@ -722,17 +722,24 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 			case OBJ_STRING_TYPES:
 				p = args [i] = alloca (ORBit_gather_alloc_info (tc));
 				do_demarshal_value (recv_buffer, &p, tc, orb);
+
+				p = args [i];
+				tprintf_trace_value (&p, tc);
 				break;
 			case STRUCT_UNION_TYPES:
 			case CORBA_tk_array:
 				if (m_data->flags & ORBit_I_COMMON_FIXED_SIZE) {
 					p = args [i] = alloca (ORBit_gather_alloc_info (tc));
 					do_demarshal_value (recv_buffer, &p, tc, orb);
+					p = args [i];
+					tprintf_trace_value (&p, tc);
 					break;
 				} /* drop through */
 			default:
 				args [i] = ORBit_demarshal_arg (
 					recv_buffer, a->tc, TRUE, orb);
+				p = args [i];
+				tprintf_trace_value (&p, tc);
 				break;
 			}
 
@@ -759,10 +766,17 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 				scratch [i] = NULL;
 				break;
 			}
+
 		}
+
+		if (m_data->arguments._buffer [i+1].flags)
+			tprintf (", ");
 	}
 
+	tprintf (")");
+
 	if (has_context) {
+		tprintf ("[FIXME:context]");
 		if (ORBit_Context_demarshal (NULL, &ctx, recv_buffer))
 			g_warning ("FIXME: handle context demarshaling failure");
 	}
@@ -776,8 +790,9 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 		goto clean_out;
 
  sys_exception:
+	/* FIXME: should we be using the connection's GIOP version ? */
 	send_buffer = giop_send_buffer_use_reply (
-		recv_buffer->connection->giop_version,
+		recv_buffer->giop_version,
 		giop_recv_buffer_get_request_id (recv_buffer),
 		ev->_major);
 
@@ -794,13 +809,20 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 				 ev->_major, ev->_id);
 			goto sys_exception;
 		}
+		tprintf ("User exception '%s'", ev->_id);
 
-	} else if (ev->_major != CORBA_NO_EXCEPTION)
+	} else if (ev->_major != CORBA_NO_EXCEPTION) {
 		ORBit_send_system_exception (send_buffer, ev);
-	
-	else { /* Marshal return values */
+
+		tprintf ("System exception");
+
+	} else { /* Marshal return values */
+		int trace_have_out = 0;
+
 		if ((tc = m_data->ret)) {
-			dprintf ("ret: ");
+			gpointer p = retval;
+
+			tprintf (" => ");
 
 			while (tc->kind == CORBA_tk_alias)
 				tc = tc->subtypes [0];
@@ -810,11 +832,13 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 			case BASE_TYPES:
 			case OBJ_STRING_TYPES:
 				ORBit_marshal_arg (send_buffer, retval, m_data->ret);
+				tprintf_trace_value (&p, tc);
 				break;
 
 			case STRUCT_UNION_TYPES:
 				if (m_data->flags & ORBit_I_COMMON_FIXED_SIZE) {
 					ORBit_marshal_arg (send_buffer, retval, m_data->ret);
+					tprintf_trace_value (&p, tc);
 					break;
 				} /* drop through */
 
@@ -822,24 +846,47 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 			case CORBA_tk_sequence:
 			case CORBA_tk_array:
 			default:
+				p = *(gpointer *) retval;
 				ORBit_marshal_arg (send_buffer, *(gpointer *)retval, m_data->ret);
+				tprintf_trace_value (&p, tc);
 				break;
 			}
 		}
 
 		for (i = 0; i < m_data->arguments._length; i++) {
 			ORBit_IArg *a = &m_data->arguments._buffer [i];
+			gpointer    p;
 			
 			tc = a->tc;
 			while (tc->kind == CORBA_tk_alias)
 				tc = tc->subtypes [0];
 			
-			if (a->flags & ORBit_I_ARG_INOUT)
+			if (a->flags & ORBit_I_ARG_INOUT) {
+				if (!trace_have_out++)
+					tprintf (" out: (");
+
 				ORBit_marshal_arg (send_buffer, args [i], tc);
+				p = args [i];
+				tprintf_trace_value (&p, tc);
+			}
+
+			else if (a->flags & ORBit_I_ARG_OUT) {
+				if (!trace_have_out++)
+					tprintf (" out: (");
 
-			else if (a->flags & ORBit_I_ARG_OUT)
 				ORBit_marshal_arg (send_buffer, scratch [i], tc);
+
+				p = scratch [i];
+				tprintf_trace_value (&p, tc);
+			}
+
+			if (trace_have_out &&
+			    m_data->arguments._buffer [i + 1].flags &
+			    (ORBit_I_ARG_OUT | ORBit_I_ARG_INOUT))
+				tprintf (", ");
 		}
+		if (trace_have_out)
+			tprintf (" )");
 	}
 
 	do_giop_dump_send (send_buffer);
@@ -871,6 +918,8 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 	}
 
  clean_out:
+	tprintf ("\n");
+
 	for (i = 0; i < m_data->arguments._length; i++) {
 		ORBit_IArg *a = &m_data->arguments._buffer [i];
 		
@@ -948,3 +997,236 @@ ORBit_small_getepv (CORBA_Object obj, CORBA_unsigned_long class_id)
 	return servant->vepv [offset];
 }
 #endif
+
+struct _ORBitAsyncQueueEntry {
+	GIOPMessageQueueEntry   mqe;
+	CORBA_Object            obj;
+	ORBitAsyncInvokeFunc    fn;
+	gpointer                user_data;
+	ORBit_IMethod          *m_data;
+	CORBA_completion_status completion_status;
+};
+
+void
+ORBit_small_demarshal_async (ORBitAsyncQueueEntry *aqe,
+			     gpointer              ret,
+			     gpointer             *args,
+			     CORBA_Environment    *ev)
+{
+	switch (orbit_small_demarshal (aqe->obj, &aqe->mqe.cnx, &aqe->mqe, ev,
+				       ret, aqe->m_data, args)) {
+	case MARSHAL_SYS_EXCEPTION_COMPLETE:
+		aqe->completion_status = CORBA_COMPLETED_YES;
+		dprintf ("Sys exception completed on id 0x%x\n\n", aqe->mqe.request_id);
+		goto system_exception;
+
+	case MARSHAL_SYS_EXCEPTION_INCOMPLETE:
+		dprintf ("Sys exception incomplete on id 0x%x\n\n", aqe->mqe.request_id);
+		goto system_exception;
+
+	case MARSHAL_EXCEPTION_COMPLETE:
+		dprintf ("Clean demarshal of exception on id 0x%x\n\n", aqe->mqe.request_id);
+		break;
+
+	case MARSHAL_RETRY:
+		g_warning ("Retry demarshal failed on id 0x%x\n\n", aqe->mqe.request_id);
+		return;
+
+	case MARSHAL_CLEAN:
+		dprintf ("Clean demarshal on id 0x%x\n\n", aqe->mqe.request_id);
+		break;
+	};
+	goto clean_out;
+
+ system_exception:
+	tprintf ("[System exception comm failure] )");
+	CORBA_exception_set_system (ev, ex_CORBA_COMM_FAILURE,
+				    aqe->completion_status);
+
+ clean_out:
+	tprintf ("\n");
+}
+
+static void
+async_recv_cb (ORBitAsyncQueueEntry *aqe)
+{
+	CORBA_Environment *ev, real_ev;
+
+	ev = &real_ev;
+	CORBA_exception_init (ev);
+
+	/* So we don't get invoked again */
+	aqe->mqe.u.unthreaded.cb = NULL;
+
+	if (aqe->mqe.cnx->parent.status == LINC_DISCONNECTED)
+		CORBA_exception_set_system (ev, ex_CORBA_COMM_FAILURE,
+					    aqe->completion_status);
+
+	aqe->fn (aqe->obj, aqe->m_data, aqe, aqe->user_data, ev);
+
+	ORBit_RootObject_release (aqe->obj);
+/*	ORBit_RootObject_release (aqe->m_data); */
+	g_free (aqe);
+	CORBA_exception_free (ev);
+}
+
+/**
+ * ORBit_small_invoke_async:
+ * @obj: 
+ * @m_data: 
+ * @fn: 
+ * @user_data: 
+ * @args: 
+ * @ctx: 
+ * @ev:
+ * 
+ *     This method is used to invoke a remote (or local) method
+ * asynchronously. @fn is called back on return - either with an empty
+ * CORBA_Environment indicating success, or with the error.
+ **/
+void
+ORBit_small_invoke_async (CORBA_Object         obj,
+			  ORBit_IMethod       *m_data,
+			  ORBitAsyncInvokeFunc fn,
+			  gpointer             user_data,
+			  gpointer            *args,
+			  CORBA_Context        ctx,
+			  CORBA_Environment   *ev)
+{
+	CORBA_unsigned_long     request_id;
+	GIOPConnection         *cnx;
+	ORBitAsyncQueueEntry   *aqe = g_new (ORBitAsyncQueueEntry, 1);
+
+	if (obj->adaptor_obj) /* a local object, we need a remote handle */
+		aqe->obj = ORBit_objref_get_proxy (obj);
+	else
+		aqe->obj = ORBit_RootObject_duplicate (obj);
+
+	cnx = ORBit_object_get_connection (aqe->obj);
+
+	if (!cnx) {
+		tprintf ("Null connection on object '%p'\n", aqe->obj);
+		aqe->completion_status = CORBA_COMPLETED_NO;
+		goto system_exception;
+	}
+
+	request_id = GPOINTER_TO_UINT (aqe);
+	aqe->completion_status = CORBA_COMPLETED_NO;
+
+	giop_recv_list_setup_queue_entry (&aqe->mqe, cnx, GIOP_REPLY, request_id);
+
+	if (! (m_data->flags & ORBit_I_METHOD_1_WAY))
+		giop_recv_list_setup_queue_entry_async (
+			&aqe->mqe, (GIOPAsyncCallback) async_recv_cb);
+	else if (fn)
+		g_warning ("oneway method being invoked async with a callback");
+
+	if (!orbit_small_marshal (aqe->obj, cnx, &aqe->mqe, request_id,
+				  m_data, args, ctx))
+		goto system_exception;
+
+	if (m_data->flags & ORBit_I_METHOD_1_WAY)
+		giop_recv_list_destroy_queue_entry (&aqe->mqe);
+
+	aqe->completion_status = CORBA_COMPLETED_MAYBE;
+	aqe->fn = fn;
+	aqe->user_data = user_data;
+	/* FIXME: perenial ORBit_IMethod lifecycle issues */
+	aqe->m_data = /* ORBit_RootObject_duplicate */ (m_data);
+
+ clean_out:
+	tprintf ("\n");
+	return;
+
+ system_exception:
+	tprintf ("[System exception comm failure] )");
+	CORBA_exception_set_system (ev, ex_CORBA_COMM_FAILURE,
+				    aqe->completion_status);
+	g_free (aqe);
+	goto clean_out;
+}
+
+gpointer
+ORBit_small_get_servant (CORBA_Object obj)
+{
+	ORBit_POAObject pobj;
+
+	if (!obj || !obj->adaptor_obj || !obj->adaptor_obj->interface)
+		return NULL;
+
+	if (obj->adaptor_obj->interface->adaptor_type != ORBIT_ADAPTOR_POA) {
+		g_warning ("Not a poa object !");
+		return NULL;
+	}
+
+	pobj = (ORBit_POAObject)obj->adaptor_obj;
+
+	return pobj ? pobj->servant : NULL;
+}
+
+static ORBitConnectionStatus
+get_status (LINCConnection *cnx)
+{
+	ORBitConnectionStatus ret;
+
+	g_return_val_if_fail (cnx != NULL, ORBIT_CONNECTION_DISCONNECTED);
+
+	switch (cnx->status) {
+	case LINC_CONNECTED:
+		ret = ORBIT_CONNECTION_CONNECTED;
+		break;
+	case LINC_CONNECTING:
+		ret = ORBIT_CONNECTION_CONNECTED;
+		break;
+	default:
+		ret = ORBIT_CONNECTION_DISCONNECTED;
+		break;
+	}
+	return ret;
+}
+
+ORBitConnectionStatus
+ORBit_small_get_connection_status (CORBA_Object obj)
+{
+	ORBitConnectionStatus ret;
+
+	if (ORBit_small_get_servant (obj))
+		ret = ORBIT_CONNECTION_IN_PROC;
+	else {
+		GIOPConnection *connection;
+
+		connection = ORBit_object_get_connection (obj);
+
+		if (connection)
+			ret = get_status (LINC_CONNECTION (connection));
+		else
+			ret = ORBIT_CONNECTION_DISCONNECTED;
+	}
+
+	return ret;
+}
+
+ORBitConnectionStatus
+ORBit_small_listen_for_broken (CORBA_Object obj,
+			       GCallback    fn,
+			       gpointer     user_data)
+{
+	ORBitConnectionStatus ret;
+
+	if (ORBit_small_get_servant (obj))
+		ret = ORBIT_CONNECTION_IN_PROC;
+	else {
+		GIOPConnection *connection;
+
+		connection = ORBit_object_get_connection (obj);
+
+		if (connection) {
+			ret = get_status (LINC_CONNECTION (connection));
+			g_signal_connect (G_OBJECT (connection),
+					  "broken", fn, user_data);
+		} else
+			ret = ORBIT_CONNECTION_DISCONNECTED;
+	}
+
+	return ret;
+}
